/**
* Klasse, die ein Formular mit einem einzigen Textfeld und evtl. einer Schaltfläche erstellt, verwaltet, validiert und Rückmeldungen anzeigt
*/
function ValidatedTextForm(
    parentNode,     // Übergeordneter DOM-Knoten
    objSettings     // Optional: Einstellungen
) {
    var self = this;
    this._form = createHtml('form', parentNode, { action: location.href, method: 'post' });
    this._label = createHtml('label', this._form);
    this._input = createHtml('input', this._label);
    this._input.addEventListener('input', function(ev) {
        self.validate();
    });
    this._form.addEventListener('submit', function(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        self.submit();
    });
    const PARAMS = [ 'text', 'placeholder', 'label', 'onvalidate', 'onsubmit', 'submitbtn' ];
    if (objSettings instanceof Object) {
        for (var z = 0; z < PARAMS.length; ++z) {
            if (objSettings[PARAMS[z]] !== undefined)
                this[PARAMS[z]] = objSettings[PARAMS[z]];
        }
    }
}

// Eingegebener Text
Object.defineProperty(ValidatedTextForm.prototype, 'text', {
    get() { return this._input.value; },
    set(s) { this._input.value = s; }
});

// Platzhaltertext
Object.defineProperty(ValidatedTextForm.prototype, 'placeholder', {
    get() { return this._input.placeholder; },
    set(s) { this._input.placeholder = s; }
});

// Beschreibung oberhalb des Eingabefeldes
Object.defineProperty(ValidatedTextForm.prototype, 'label', {
    get() {
        if (this._label.firstChild.nodeName.toUpperCase() == 'SPAN')
            return this._label.firstChild.textContent;
        else
            return undefined;
    },
    set(s) {
        if ((typeof s == 'string') || (s instanceof String)) {
            if (this._label.firstChild.nodeName.toUpperCase() == 'SPAN')
                this._label.firstChild.textContent = s;
            else
                createHtml('span', this._label, null, s, this._label.firstChild);
        } else {
            if (this._label.firstChild.nodeName.toUpperCase() == 'SPAN')
                this._label.removeChild(this._label.firstChild);
        }
    }
});

// Schaltfläche zum Absenden
Object.defineProperty(ValidatedTextForm.prototype, 'submitbtn', {
    get() {
        if (this._form.lastChild.nodeName.toUpperCase() == 'BUTTON')
            return this._form.lastChild.value;
        else
            return false;
    },
    set(s) {
        if (s || (typeof s == 'string') || (s instanceof String)) {
            if (this._form.lastChild.nodeName.toUpperCase() == 'BUTTON')
                this._form.lastChild.textContent = (s === true) ? 'OK' : s;
            else
                createHtml('button', this._form, { type: 'submit' }, s);
        } else {
            if (this._form.lastChild.nodeName.toUpperCase() == 'BUTTON')
                this._form.removeChild(this._form.lastChild);
        }
    }
});

// Funktion, welche Validierung durchführt
ValidatedTextForm.prototype.onvalidate = null;

// Ergebnis der letzten Validierung
ValidatedTextForm.prototype.isValid = true;

// Anzuzeigendes Ergebnis der letzten Validierung
ValidatedTextForm.prototype.strMessage = '';

/**
* Textfeld fokusieren
* @return undefined
*/
ValidatedTextForm.prototype.focus = function() {
    this._input.focus();
}

/**
* Ganzen Text im Textfeld markieren
* @return undefined
*/
ValidatedTextForm.prototype.select = function() {
    this._input.select();
}

/**
* Angezeigte Nachricht aktualisieren
* @return undefined
*/
ValidatedTextForm.prototype.updateMessage = function() {
    this._input.setCustomValidity(this.isValid ? '' : this.strMessage);
    /*!*/
}

/**
* Eingegebenen Text validieren
* @return Boolean   Ist die Eingabe valide?
*/
ValidatedTextForm.prototype.validate = function() {
    if (typeof this.onvalidate == 'function') {
        this.isValid = this.onvalidate.call(this, this.text);
    } else {
        this.isValid = true
        this.strMessage = '';
    }
    this.updateMessage();
    return this.isValid;
}

// Was soll beim Absenden des Formulars passieren?
ValidatedTextForm.prototype.onsubmit = null;

/**
* Absenden des Formulars verarbeiten
* @return undefined
*/
ValidatedTextForm.prototype.submit = function() {
    if (this.validate() && (typeof this.onsubmit == 'function'))
        this.onsubmit.call(this, this.text);
}
